#ifndef __IUF_AST3DFRACT
#define __IUF_AST3DFRACT
/** This class represent a asteroid with a 3D random fractal shape.
The fractal shape is stored in a mesh and it is generated, starting 
from a loaded base mesh (i.e. ast.raw or tetra.raw) and
by recursively subdividing each face into four smaller faces.
*/

#include "Triangle.h"

class Ast3DFract: public Ast
{ 
public:

	/** The mesh representing the asteroid shape. */
	Mesh M;

/** Default constructor.
It load a mesh from a file (default is tetra.raw, a simple tetrahedron), 
resize it into a unitary box and \Ref{Split} each face three times (e.g. 
the resulting mesh has 64x faces). At the end it calculates smooth per-vertex normal 
Version History {\small 
\begin{itemize}
\item 0.4 Number of split depend on ast size.
\end{itemize}
}

*/
	//Ast3DFract(char *filename="ast.raw")
	Ast3DFract(char *filename="tetra.raw",float _size=5){
	M.Open(filename);
	M.Normalize();
	Split(rand()+glutGet(GLUT_ELAPSED_TIME),1);
	//Split(rand()+glutGet(GLUT_ELAPSED_TIME),0.5);
	if(_size>SmallSize) Split(rand()+glutGet(GLUT_ELAPSED_TIME),0.5);
	if(_size>MediumSize) Split(rand()+glutGet(GLUT_ELAPSED_TIME),0.5);
	M.Smooth();
	size=_size;
	B.r=size;
	}

/** 
Draw the asteroid shape.
Simply draw the stored mesh.
Version History {\small 
\begin{itemize}
\item 0.4 Now it uses display lists.
\end{itemize}
}
*/
virtual	void Draw();

/** This function split each triangle of the mesh into four smaller triangles.
When splitting a triangle we geneate three new vertices (one for each edge) whose coordinates are 
calculated by \Ref{MidPoint} depending on the coordinates of the two 
vertices of the triangle edge where the new midpoint lies.
The two params are passed directly to the \Ref{MidPoint} 
function and influence how the new point is calculated.
*/
void Split(int seed, float strenght=1);

/** This function calculates a point on an edge that is slightly perturbed form its original midpoint position.
It is important that the pertubation is not really random but it depends on the coords of the two edge vertexes.
In this way we generate twice the same midpoint when splitting the same edge on two adjacent triangles.
@param p0_p1 the two points of the edge. We use an hash function to get a random seed for each vertex.
@param BC the BariCenter of the Asteroids; the perturbation is along the line from bc to the middlepoint
@param seed a random seed to avoid to calculate always the same asteroid when starting from the same triangles...
@param stenght the {strenght} of the perturbation when strenght == 0 the shape tends to a sphere, good values are in the [.2,2] range.

*/
Point3f MidPoint( Point3f  &p0,Point3f  &p1,  Point3f const &BC,int seed,float strengh);
};
#endif